import sys
import logging

from alphatwirl import concurrently, progressbar
from alphatwirl.misc.deprecation import _deprecated

from alphatwirl.parallel import Parallel
from .SGEJobSubmitter import SGEJobSubmitter

##__________________________________________________________________||
def build_parallel(parallel_mode, quiet=True, processes=4, user_modules=[ ],
                   dispatcher_options=[ ]):

    dispatchers = ('subprocess', 'htcondor', 'sge')
    parallel_modes = ('multiprocessing', ) + dispatchers
    default_parallel_mode = 'multiprocessing'

    if not parallel_mode in parallel_modes:
        logger = logging.getLogger(__name__)
        logger.warning('unknown parallel_mode "{}", use default "{}"'.format(
            parallel_mode, default_parallel_mode
        ))
        parallel_mode = default_parallel_mode

    if parallel_mode == 'multiprocessing':
        return _build_parallel_multiprocessing(quiet=quiet, processes=processes)

    return build_parallel_dropbox(
        parallel_mode=parallel_mode,
        user_modules=user_modules,
        dispatcher_options=dispatcher_options,
    )

##__________________________________________________________________||
def build_parallel_dropbox(parallel_mode, user_modules,
                           dispatcher_options=[ ]):
    workingarea_topdir = '_ccsp_temp'
    python_modules = set(user_modules)
    python_modules.add('alphatwirl')
    workingarea_options = dict(topdir=workingarea_topdir, python_modules=python_modules)

    if parallel_mode == 'htcondor':
        dispatcher_options = dict(job_desc_extra=dispatcher_options)
        dispatcher_class = concurrently.HTCondorJobSubmitter
        dropbox_options = dict()
    elif parallel_mode == 'sge':
        dispatcher_options = dict()
        dispatcher_class = SGEJobSubmitter
        dropbox_options = dict(sleep = 30)
    else:
        dispatcher_options = dict()
        dispatcher_class = concurrently.SubprocessRunner
        dropbox_options = dict()

    return _build_parallel_dropbox_(
        workingarea_options, dropbox_options, dispatcher_class, dispatcher_options
    )

def _build_parallel_dropbox_(workingarea_options, dropbox_options,
                             dispatcher_class, dispatcher_options):

    workingarea = concurrently.WorkingArea(**workingarea_options)

    dispatcher = dispatcher_class(**dispatcher_options)

    dropbox_options.update(dict(workingArea=workingarea, dispatcher=dispatcher))
    dropbox = concurrently.TaskPackageDropbox(**dropbox_options)
    communicationChannel = concurrently.CommunicationChannel(dropbox=dropbox)

    progressMonitor = progressbar.NullProgressMonitor()

    return Parallel(progressMonitor, communicationChannel, workingarea)

##__________________________________________________________________||
def _build_parallel_multiprocessing(quiet, processes):

    if quiet:
        progressBar = None
    elif sys.stdout.isatty():
        progressBar = progressbar.ProgressBar()
    else:
        progressBar = progressbar.ProgressPrint()

    if processes is None or processes == 0:
        progressMonitor = progressbar.NullProgressMonitor() if quiet else progressbar.ProgressMonitor(presentation = progressBar)
        communicationChannel = concurrently.CommunicationChannel0()
    else:
        progressMonitor = progressbar.NullProgressMonitor() if quiet else progressbar.BProgressMonitor(presentation = progressBar)
        dropbox = concurrently.MultiprocessingDropbox(processes, progressMonitor)
        communicationChannel = concurrently.CommunicationChannel(dropbox = dropbox)

    return Parallel(progressMonitor, communicationChannel)

##__________________________________________________________________||

##__________________________________________________________________||
@_deprecated(msg='use alphatwirl.parallel.build.build_parallel() instead.')
def build_parallel_multiprocessing(quiet, processes):
    return build_parallel(
        parallel_mode='multiprocessing',
        quiet=quiet, processes=processes
    )

##__________________________________________________________________||



import setuptools

def get_long_description():
    with open("README.md", 'r') as fh:
        long_description = fh.read()
    return long_description

setuptools.setup(
    name="atsge",
    version="0.1.5",
    author="Shane Breeze",
    author_email="sdb15@ic.ac.uk",
    scripts=[],
    description="SGE batch job submission with AlphaTwirl",
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/shane-breeze/atsge",
    packages=setuptools.find_packages(),
    download_url="https://github.com/shane-breeze/atsge/archive/0.1.5.tar.gz",
    install_requires=["alphatwirl>=0.20.1"],
    setup_requires=["pytest-runner"],
    tests_require=["pytest"],
    classifiers=[
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Utilities",
        "Development Status :: 3 - Alpha",
    ],
)
